;=============================================================================
; Insight, real-mode debugger for MS DOS / PC DOS / FreeDOS.
; Copyright (c) Victor M. Gamayunov, Sergey Pimenov, 1993, 96, 97, 2002.
; Modifications by Oleg O. Chukaev (2006 - 2009).
;-----------------------------------------------------------------------------
; keyboard.inc
; Procedures for working with keyboard.
;-----------------------------------------------------------------------------
; This program is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public License
; as published by the Free Software Foundation; either version 2
; of the License, or (at your option) any later version.
;
; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
; 
; You should have received a copy of the GNU General Public License
; along with this program; if not, write to the Free Software
; Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
; 02110-1301 USA
;=============================================================================


;=============================================================================
; Changelog
;-----------------------------------------------------------------------------
; 2007-02-10	0leg	Added comments to all procedures. All variables moved
;			to act[u]data.inc
;=============================================================================


;=============================================================================
; key_loop
;-----------------------------------------------------------------------------
; Reads keystrokes and calls corresponding procedures in loop.
; On end of table key_loop calls corresponding proc, if it is not 0.
; If it is 0 or if it return CY, [test_key] called. If [test_key]
; return NC, key_loop terminates, otherwise beep called.
; In:	[keys] -> structure: key-code: word, proc: word,
;		end of table: key-code == 0, exit: proc == 0
; Out:	AX -- key codes
; Modf:	AX, BX, CX, DX, SI, DI, BP
; Call:	flush_key, read_key, [test_key], [key_loop_sub], beep
; Use:	keys, key_loop_sub, cur_test_code, test_key
;
key_loop:

@@loop:
		push	ax
		push	bx
		push	si
		mov	si,[keys]
		call	flush_key
		call	read_key
		xchg	ax,bx
@@next:
		lodsw
		or	ax,ax			;End of table?
		jz	@@not_found		;Yes
		cmp	ax,bx
		je	@@found
		lodsw
		jmp	@@next

@@found:
		lodsw
		or	ax,ax			;Exit?
		jz	@@quit			;Yes
		mov	[key_loop_sub],ax
		pop	si
		pop	bx
		pop	ax
		call	word [key_loop_sub]	;Call procedure
		jmp	@@loop

@@not_found:
		mov	[cur_test_code],bx	;Key codes
		lodsw
		mov	[key_loop_sub],ax	;Procedure for any key
		or	ax,ax			;Exit?
		xchg	ax,bx
		pop	si
		pop	bx
		jz	@@test_key		;Yes
		call	word [key_loop_sub]	;AX == codes on entry
		jc	@@test_key
		pop	ax
		jmp	@@loop			;Next...

@@test_key:
		pop	ax
		call	word [test_key]
		jnc	@@quit_1
		call	beep
		jmp	@@loop
@@quit:
		mov	ax,bx			;Key codes
		pop	si
		pop	bx
		inc	sp
		inc	sp
@@quit_1:
		ret
;=============================================================================
; def_test_key
;-----------------------------------------------------------------------------
; Default key testing procedure.
; In:	---
; Out:	CY
; Modf:	---
; Call:	---
; Use:	---
;
def_test_key:
		stc
		ret
;=============================================================================
; read_key
;-----------------------------------------------------------------------------
; Reads keystroke.
; In:	---
; Out:	AX -- ASCII/scan codes
; Modf:	AX
; Call:	---
; Use:	---
;
read_key:
	xor ax, ax
	xchg ax, word [cs:read_key_insert]
	test ax, ax
	jnz .ret
		push	bx
		xor	bx,bx
	jmp @F
@@next:
	call terminal_receive_to_buffer
	call idle			; idle while waiting
@@:
		mov	ah,1
		int	16h
		jnz	@@get
		mov	ah,2
		int	16h
		test	al,8
		jnz	@@alt
		cmp	bl,2
		je	@@ret
		mov	bl,1
		jmp	@@next
@@ret:
		mov	ax,kbAltAlone
		jmp	@@quit
@@get:
		mov	ah,0
		int	16h
@@quit:
		pop	bx
.ret:
		ret
@@alt:
		or	bx,bx
		jz	@@next
		mov	bl,2
		jmp	@@next

terminal_receive_to_buffer:
	call pushr
	push cs
	pop ds
	rol byte [terminal_connected], 1
	jnc .shownix
.loop:
	push cs
	pop es
		call IRQ_READBUF        ; ***** fetch data from input buffer
	test ax, ax
	js .shownix	; only show properly received chars
			; (do not show the -1 we get if no chars received :-))
	mov di, word [terminal_buffer.end]
	mov es, word [terminal_buffer]
	mov ah, byte [es:di]
	stosb
	inc word [terminal_buffer.end]
	 mov bx, word [terminal_buffer.draw]
	cmp di, word [terminal_buffer.start]
	jne .leave
	push ax
	mov si, word [terminal_buffer.start]
	call .inc_bx_if_equal
	inc si
	xchg ah, al
@@:
	cmp al, 10
	je @F
	 call .inc_bx_if_equal
	es lodsb
	jmp @B

@@:
	cmp word [terminal_buffer.draw], bx
	je @F
	dec word [terminal_buffer.lf_amount]
@@:
	 mov word [terminal_buffer.draw], bx
	mov word [terminal_buffer.start], si
	pop ax
.leave:

	cmp al, 10
	jne .not_write_lf
	cmp word [terminal_buffer.lf_amount], MAX_SCR_ROWS
	jbe .increment_lf_amount
	xchg bx, si
@@:
	cmp si, word [terminal_buffer.end]
	je @F				; (unlikely, we had lf_amount > rows)
	es lodsb
	cmp al, 10
	jne @B
@@:
	mov al, 10
	mov word [terminal_buffer.draw], si
	jmp .not_write_lf

.increment_lf_amount:
	inc word [terminal_buffer.lf_amount]

.not_write_lf:
	rol byte [terminal_active], 1
	jnc .loop
	mov es, [video_seg]
	call SHOW_RX
	jmp .loop
.shownix:
	call popr
	retn

.inc_bx_if_equal:
	cmp si, bx
	jne @F
	inc bx
@@:
	retn

;=============================================================================
; in_key
;-----------------------------------------------------------------------------
; Checks if key pressed.
; In:	---
; Out:	ZR if no key pressed
;	NZ if key pressed:
;		AX -- ASCII/scan codes
; Modf:	AX
; Call:	---
; Use:	---
;
%if 0
in_key:

		mov	ah,1
		int	16h
		ret
%endif
;=============================================================================
; flush_key
;-----------------------------------------------------------------------------
; Flushes keyboard buffer.
; In:	---
; Out:	---
; Modf:	AX
; Call:	---
; Use:	---
;
flush_key:
@@next:
		mov	ah,1
		int	16h
		jz	@@quit
		mov	ah,0
		int	16h
		jmp	@@next
@@quit:
		ret
;=============================================================================
; check_shift
;-----------------------------------------------------------------------------
; Checks if any shift key is pressed.
; In:	---
; Out:	ZR if shift key is not pressed
;	NZ if shift key is pressed
; Modf:	---
; Call:	---
; Use:	---
;
check_shift:
		push	ax
		push	ds
		xor	ax,ax
		mov	ds,ax
		test	byte [417h],11b
		pop	ds
		pop	ax
		ret


		; INP:	-
		; OUT:	-
		; CHG:	ax
		; STT:	ds = ss = debugger segment/selector
		;
		; Idle system, using 2F.1680 or sti \ hlt.
idle:
	; testopt [options3], opt3_no_idle_2F
	; jnz .hlt
	rol byte [option_1680], 1
	jnc .hlt
 ; %if _GUARD_86M_INT2F
 %if 1
	push es
	xor ax, ax
	mov es, ax		; (only used in 86 Mode)
	mov ax, [es:2Fh * 4]
	cmp ax, -1
	je @F			; --> (ZR)
	or ax, [es:2Fh * 4 + 2]
@@:
	pop es
	jz @FF
@@:
 %endif
	mov ax, 1680h
	int 2Fh			; release timeslice in multitasker
	test al, al
	jz .return		; done idling -->
@@:
.hlt:
	; testopt [options], nohlt
	; jnz .return
	rol byte [option_hlt], 1
	jnc .return
	sti
	hlt			; else idle by hlt
	nop
.return:
	retn

;=============================================================================
; E0F
;=============================================================================
